#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for ticker() method"""

from pymarketcap.tests import type_test
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def assert_types(res):
    map_types = {
        "name":               str,
        "symbol":             str,
        "added":              str,
        "market_cap":         (float, type(None)),
        "price":              float,
        "circulating_supply": (float, type(None)),
        "volume_24h":         (float, type(None)),
        "percent_change":     (float, type(None)),
    }

    assert isinstance(res, list)
    assert len(res) > 0
    for currency in res:
        assert isinstance(currency, dict)
        for key, value in currency.items():
                type_test(map_types, key, value)

def test_types():
    assert_types(pym.recently())

def test_consistence():
    res = list(pym.recently())

def test_convert():
    res_btc = pym.recently(convert="BTC")

    # Test types
    assert_types(res_btc)

    # Test consistence
    res_usd = pym.recently()
    for curr_btc, curr_usd in zip(res_btc, res_usd):
        for key in curr_btc:
            assert type(curr_btc[key]) == type(curr_usd[key])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Warning: use this example with caution,
it will use a lot of your memory."""

import asyncio
from pymarketcap import AsyncPymarketcap

res = []

async def main():
    async with AsyncPymarketcap(queue_size=50, consumers=50) as apym:
        async for currency in apym.every_historical():
            res.append(currency)

if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())

    print(res[-1])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
import asyncio
import pytest

sys.path.append(os.path.join(os.getcwd(), "pymarketcap"))

def pytest_addoption(parser):
    parser.addoption("--end2end", action="store_true",
                     default=False, help="Run slower tests.")

def pytest_collection_modifyitems(config, items):
    if config.getoption("--end2end"):
        # Run end2end tests
        return
    skip_end2end = pytest.mark.skip(reason="Need --end2end option to run")
    for item in items:
        if "end2end" in item.keywords:
            item.add_marker(skip_end2end)

@pytest.yield_fixture()
def event_loop():
    loop = asyncio.get_event_loop()
    yield loop
    #loop.close()
    pass





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice
from re import findall
from urllib.request import urlopen

import pytest

from pymarketcap.tests.markets import (
    assert_types,
    assert_consistence
)
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def assert_number_of_markets(res):
    req = urlopen("https://coinmarketcap.com/currencies/%s" % res["slug"])
    data = req.read()
    req.close()
    indexes = findall(r'<td class="text-right">(.+)</td>', data.decode())
    assert len(res["markets"]) in [
        int(indexes[-1])+1,
        int(indexes[-1]),
        int(indexes[-1])-1
    ]

def test_without_convert():
    coin = choice(pym.coins)
    print("(Currency: %s)" % coin, end=" ")
    res = pym.markets(coin)
    assert_types(res)
    assert_consistence(res)
    assert_number_of_markets(res)

def test_with_convert():
    coin = choice(pym.coins)
    print("(Currency: %s)" % coin, end=" ")
    res = pym.markets(coin, convert="BTC")
    assert_types(res)
    assert_consistence(res)
    assert_number_of_markets(res)

def test_invalid():
    symbol = "BDAD)DAAS&/9324423OUVibb"
    with pytest.raises(ValueError) as excinfo:
        res = pym.markets(symbol)
    assert "See 'symbols' or 'coins' properties" in str(excinfo)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""``markets()`` shared method test module."""

from pymarketcap.tests import type_test

def assert_types(res):
    map_types = {
        "source":         str,
        "pair":           str,
        "volume_24h":     float,
        "price":          float,
        "percent_volume": float,
        "updated":        bool,
    }

    assert isinstance(res["markets"], list)
    assert isinstance(res["symbol"], str)
    assert isinstance(res["slug"], str)
    for source in res["markets"]:
        assert isinstance(source, dict)
        for key, value in source.items():
            type_test(map_types, key, value)

def assert_consistence(res):
    for source in res["markets"]:
        assert len(source.keys()) == 6
        assert source["pair"].count("/") == 1
        assert source["percent_volume"] <= 100



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pytest

from pymarketcap.tests.exchange import (
    assert_types,
    assert_consistence
)
from pymarketcap import (
    AsyncPymarketcap,
    Pymarketcap
)
pym = Pymarketcap()


@pytest.mark.end2end
def test_every_exchange(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(queue_size=50,
                                    consumers=50) as apym:
            res = []
            show_msg = True
            async for exc in apym.every_exchange():
                if show_msg:
                    print("Testing all responses...")
                    show_msg = False
                res.append(exc)

                assert_types(exc)
                assert_consistence(exc)
            assert type(res) == list
    event_loop.run_until_complete(wrapper())






#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice
from re import findall
from urllib.request import urlopen

from tqdm import tqdm
import pytest

from pymarketcap.tests.exchange import (
    assert_types,
    assert_consistence
)
from pymarketcap import Pymarketcap, CoinmarketcapHTTPError
pym = Pymarketcap()

def assert_number_of_markets(res):
    req = urlopen("https://coinmarketcap.com/exchanges/%s/" % res["slug"])
    data = req.read()
    req.close()
    indexes = findall(r'<td class="text-right">(\d+)</td>', data.decode())
    try:
        assert len(res["markets"]) == int(indexes[-1])
    except IndexError:
        if req.getcode() != 200:  # HTTP fail
            pass

def test_types():
    exc = choice(pym.exchange_slugs)
    print("(Exchange: %s)" % exc, end=" ")
    res = pym.exchange(exc)
    assert_types(res)

def test_consistence():
    exc = choice(pym.exchange_slugs)
    print("(Exchange: %s)" % exc, end=" ")
    res = pym.exchange(exc)
    assert_consistence(res)
    assert_number_of_markets(res)


def test_convert():
    exc = choice(pym.exchange_slugs)
    print("(Exchange: %s)" % exc, end=" ")
    res = pym.exchange(exc, convert="BTC")

    assert_types(res)
    assert_consistence(res)

def test_invalid():
    exc = "dabsfgbdsagubfeqbfeyfv"
    with pytest.raises(ValueError) as excinfo:
        pym.exchange(exc)
    assert '%s is not a valid exchange name.' % exc in str(excinfo.value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Test wrapper properties"""

from pymarketcap.consts import INVALID_COINS
from pymarketcap import Pymarketcap
pym = Pymarketcap()


###   API METHOD CACHERS   ###

def test_correspondences():
    res = pym.correspondences

    # Test types
    assert type(res) == dict
    for key, value in res.items():
        assert type(key) == str
        assert type(value) == str

    # Test consistence
    assert res == pym._cache_symbols_ids()[0]

def test_ids_correspondences():
    res = pym.ids_correspondences

    # Test types
    assert type(res) == dict
    for key, value in res.items():
        assert type(key) == str
        assert type(value) == int

    # Test consistence
    assert res == pym._cache_symbols_ids()[1]

def test_symbols():
    res = pym.symbols

    # Test types
    assert type(res) == list

    # Test consistence
    assert len(res) > 0

def test_coins():
    res = pym.coins

    # Test types
    assert type(res) == list
    for coin in res:
        assert type(coin) == str

    # Test consistence
    for invalid_coin in INVALID_COINS:
        assert invalid_coin not in res
    assert len(res) > 0

def test_total_currencies():
    # Test types
    res = pym.total_currencies
    assert type(res) == int

    # Test consistence
    assert res == len(pym.ticker())


###   WEB SCRAPER METHOD CACHERS   ###

def test_currency_exchange_rates():
    # Test types
    res = pym.currency_exchange_rates
    for key, value in res.items():
        assert type(key) == str
        assert type(value) == float

    # Test consistence
    ctc = pym.currencies_to_convert
    for symbol in list(res.keys()):
        assert symbol in ctc

def test_currencies_to_convert():
    # Test types
    currencies = pym.currencies_to_convert
    assert type(currencies) == list
    for curr in currencies:
        assert type(curr) == str

    # Test consistence
    assert len(currencies) > 0

def test_exchange_names():
    # Test types
    res = pym.exchange_names
    assert type(res) == list
    assert len(res) > 0
    for exc in res:
        assert type(exc) == str

    # Test consistence
    assert len(res) > 0
    assert len(res) == len(pym.__exchange_names_slugs())

def test_exchange_slugs():
    # Test types
    res = pym.exchange_slugs
    assert type(res) == list
    for exc in res:
        assert type(exc) == str

    # Test consistence
    assert len(res) > 0
    assert len(res) == len(pym.__exchange_names_slugs())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for ticker() method"""

from time import sleep
from random import choice, randint

from pymarketcap.tests import type_test
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def teardown_function():
    sleep(1)

def assert_types(res):
    map_types = {
        "id":                   str,
        "name":                 str,
        "symbol":               str,
        "rank":                 int,
        "price_usd":            (float, type(None)),
        "price_btc":            (float, type(None)),
        "24h_volume_usd":        (float, type(None)),
        "market_cap_usd":       (float, int, type(None)),
        "available_supply":     (float, int, type(None)),
        "total_supply":         (float, int, type(None)),
        "max_supply":           (float, int, type(None)),
        "percent_change_1h":    (float, type(None)),
        "percent_change_24h":   (float, type(None)),
        "percent_change_7d":    (float, type(None)),
        "last_updated":         (int, type(None))
    }
    assert isinstance(res, (list, dict))

    for key, value in res.items():
        if key in map_types:
            type_test(map_types, key, value)
        else:
            assert isinstance(value, (float, int, type(None)))


def test_consistence():
    res = pym.ticker()
    assert res[0]["rank"] == 1
    assert res[-1]["rank"] == pym.total_currencies

def test_limit():
    res = pym.ticker(limit=40)
    assert len(res) == 40
    assert res[0]["rank"] == 1
    res = pym.ticker(limit=385)
    assert len(res) == 385

def test_name():
    symbol = choice(pym.symbols)
    res = pym.ticker(symbol)
    print("(Symbol: %s)" % symbol, end=" ")

    assert_types(res)

def test_start():
    pos = randint(0, pym.total_currencies - 1)
    res = pym.ticker(start=pos)
    assert res[0]["rank"] == (pos + 1)

def test_convert():
    for currency in [True, False]:  # With and without currency
        symbol = None
        badge = choice(pym.ticker_badges)
        if currency:
            symbol = choice(pym.symbols)
            print("(Currency: %s - Badge: %s)" % (symbol, badge), end=" | ")
        else:
            print("(Badge: %s)" % badge, end=" ")

        res = pym.ticker(currency=symbol, convert=badge)
        keys = res.keys() if currency else res[0].keys()
        badges_in_keys_count = 0
        for key in keys:
            if badge.lower() in key:
                badges_in_keys_count += 1
        assert badges_in_keys_count == 3

        if currency:
            assert_types(res)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pytest

from pymarketcap import AsyncPymarketcap

def test_types(event_loop):
    async def wrapper():
        async with AsyncPymarketcap(debug=True) as apym:
            res = await apym._cache_symbols()

            assert type(res) == dict
            for symbol, slug in res.items():
                assert type(symbol) == str
                assert type(slug) == str
    event_loop.run_until_complete(wrapper())


def test_consistence(event_loop):
    async def wrapper():
        async with AsyncPymarketcap() as apym:
            res = await apym._cache_symbols()
            assert len(res) > 0
    event_loop.run_until_complete(wrapper())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice, randint
from datetime import datetime, timedelta

from pymarketcap.tests.graphs import assert_types
from pymarketcap import Pymarketcap
pym = Pymarketcap()

all_symbols = pym.symbols

def test_types():
    symbol = choice(all_symbols)
    res = pym.graphs.currency(symbol)
    print("(Currency: %s)" % symbol, end=" ")

    assert_types(res)

def test_consistence():
    symbol = choice(all_symbols)
    res = pym.graphs.currency(symbol)
    print("(Currency: %s)" % symbol, end=" ")

    assert len(res) in [4, 5]

    fields = [
        "market_cap_by_available_supply",
        "price_btc",
        "price_usd",
        "volume_usd",
        "price_platform",
     ]

    for field, values in res.items():
        assert field in fields

def test_start_end():
    days_ago = timedelta(days=randint(1, 15))
    start = datetime.now() - days_ago
    end = datetime.now()
    symbol = choice(all_symbols)
    print("(Currency: %s | Start: %r)" % (symbol, start), end=" ")
    for startend in [True , False]:
        params = {"start": start, "end": end}
        if not startend:
            params["start"] = None
            params["end"] = None
        res = pym.graphs.currency(choice(pym.symbols), **params)

        # Test types
        assert type(res) == dict

        for key, _list in res.items():
            assert type(_list) == list
            for elem, value in _list:
                assert isinstance(elem, datetime)
                assert type(value) in [int, float]

        # Test consistence
        assert end.day >= res["price_btc"][-1][0].day
        if startend:
            first_day = res["price_usd"][0][0]
            next_day = res["price_usd"][0][0] + timedelta(days=1)
            assert start <= next_day








from libc.stddef cimport size_t
from libc.string cimport memcpy
from cpython.mem cimport PyMem_Malloc, PyMem_Realloc, PyMem_Free
from curl cimport *

from pymarketcap.errors import CoinmarketcapHTTPError
from pymarketcap import __version__

# https://curl.haxx.se/libcurl/c/getinmemory.html
cdef struct MemoryStruct:
    char *memory
    size_t size

cdef size_t write_memory(void *contents, size_t size, size_t nmemb, void *userp):
    cdef size_t realsize = size * nmemb
    cdef MemoryStruct *mem = <MemoryStruct *>userp
    mem.memory = <char *>PyMem_Realloc(mem.memory, mem.size + realsize + 1);
    if mem.memory == NULL:
        print("Not enough memory (realloc returned NULL)\n")
        return 0
    memcpy(&(mem.memory[mem.size]), contents, realsize);
    mem.size += realsize;
    mem.memory[mem.size] = 0;

    return realsize

cdef class Response(object):
    cdef readonly long status_code
    cdef readonly bytes text
    cdef readonly bytes content_type
    cdef readonly bytes encoding
    cdef readonly bytes url

    def __cinit__(self, text):
        self.text = text

cpdef Response get_to_memory(const char *url, long timeout, bint debug):
    cdef CURLcode ret
    cdef long true = 1L
    version = curl_version()
    cdef CURL *curl = curl_easy_init()
    cdef const char *user_agent = "pymarketcap 3.9.161"
    cdef const char *accept_encoding = "gzip, deflate"
    cdef char *raw_body

    cdef MemoryStruct chunk
    chunk.memory = <char *>PyMem_Malloc(1)
    chunk.size = 0

    if curl != NULL:
        if debug:
            curl_easy_setopt(curl, CURLOPT_VERBOSE, &true)
        curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, &true)
        ret = curl_easy_setopt(curl, CURLOPT_URL, url)
        ret = curl_easy_setopt(curl, CURLOPT_TIMEOUT, <void *>timeout)

        ret = curl_easy_setopt(curl, CURLOPT_HTTPGET, &true)

        ret = curl_easy_setopt(curl, CURLOPT_USERAGENT, user_agent)
        ret = curl_easy_setopt(curl, CURLOPT_ACCEPT_ENCODING, accept_encoding)

        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, &write_memory)
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, <void *>&chunk)

        if ret != CURLE_OK:
            raise RuntimeError
        ret = curl_easy_perform(curl)
        if ret != CURLE_OK:
            raise CoinmarketcapHTTPError

        resp = Response(chunk.memory)
        curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &resp.status_code)
        curl_easy_cleanup(curl)

        PyMem_Free(chunk.memory)

        return resp
